# Generated by Django 1.11.5 on 2018-05-24 22:04


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0004_auto_20180508_1652'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterpriseenrollment',
            name='enterprise_id',
            field=models.UUIDField(),
        ),
    ]
