# Generated by Django 4.2.16 on 2025-01-08 07:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0045_alter_enterpriseexecedlcmoduleperformance_options_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='EnterpriseGroupMembership',
            fields=[
                ('group_membership_unique_id', models.CharField(max_length=64, primary_key=True, serialize=False)),
                ('is_applies_to_all_contexts', models.BooleanField(default=False)),
                ('enterprise_customer_id', models.UUIDField(db_index=True, null=True)),
                ('enterprise_group_name', models.CharField(max_length=255, null=True)),
                ('enterprise_group_uuid', models.UUIDField(null=True)),
                ('group_is_removed', models.BooleanField(default=False)),
                ('group_type', models.CharField(max_length=128, null=True)),
                ('activated_at', models.DateTimeField(null=True)),
                ('enterprise_customer_user_id', models.PositiveIntegerField(null=True)),
                ('membership_is_removed', models.BooleanField(default=False)),
                ('membership_status', models.CharField(max_length=128, null=True)),
                ('enterprise_group_membership_uuid', models.UUIDField(null=True)),
            ],
            options={
                'verbose_name': 'Group Membership',
                'verbose_name_plural': 'Group Memberships',
                'db_table': 'group_membership',
                'indexes': [models.Index(fields=['enterprise_group_uuid', 'group_type', 'enterprise_customer_user_id'], name='group_membe_enterpr_796f48_idx')],
            },
        ),
    ]
