"""
This module contains the database queries for the admin analytics.
"""
from .fact_engagement_admin_dash import FactEngagementAdminDashTable
from .fact_enrollment_admin_dash import FactEnrollmentAdminDashTable
from .skills_daily_rollup_admin_dash import SkillsDailyRollupAdminDashTable
