"""
Constants for enterprise data app.
"""


ENTERPRISE_DATA_ADMIN_ROLE = 'enterprise_data_admin'
SYSTEM_ENTERPRISE_ADMIN_ROLE = 'enterprise_admin'
SYSTEM_ENTERPRISE_OPERATOR_ROLE = 'enterprise_openedx_operator'

# this constant should not be used anymore as it is marked for removal
# switch is used to enable/disable role based access control
ROLE_BASED_ACCESS_CONTROL_SWITCH = 'role_based_access_control'

# context to give access to all resources
ALL_ACCESS_CONTEXT = '*'
